#include <GL/glut.h>
#include <stdlib.h>

#include "GameObj.h"
#include "Bullet.h"
#include "Ship.h"
#include "TriShip.h"
#include "Ship3D.h"
#include "Ast.h"
#include "AstFract.h"
#include "Ast3DFract.h"
#include "Exp.h"
#include "GameSession.h"

#ifdef _WIN32
#define drand48() (((float) rand())/((float) RAND_MAX))
#define srand48(x) (srand((x)))
#endif

/** The main drawing function.
It draw the current ship, all the active bullets,
asteroids and  generic objects
*/
void GameSession::Draw()
{
	if(S->active)
		S->Draw();

	list<Bullet *>::iterator lib;
	for(lib=ActiveBullet.begin();lib!=ActiveBullet.end();++lib)
		(*lib)->Draw();
	list<Ast *>::iterator lia;
	for(lia=ActiveAst.begin();lia!=ActiveAst.end();++lia)
		(*lia)->Draw();
	list<GameObj *>::iterator li;
	for(li=ActiveObj.begin();li!=ActiveObj.end();++li)
		(*li)->Draw();	
}

void GameSession::Update(int dt)
{
	if(PauseFlag) return;
	if(S->active){
		S->Update(dt);
		Clip(S->p);
	} else {
		deadpause-=dt;
		if(deadpause<0){
			S->active=true;
			S->p=Point3f(0,0,0);
			S->v=Point3f(0,0,0);
		}
	}

	list<Bullet *>::iterator lib;
	for(lib=ActiveBullet.begin();lib!=ActiveBullet.end();++lib)
		{
		 (*lib)->Update(dt);
			Clip((*lib)->p);
			BulletAstCollision(*lib);
			if((*lib)->active==false)
			{
				GameObj *todel=*lib;
				lib=ActiveBullet.erase(lib);
				S->TerminateBullet();
				delete todel;
			}
			if(lib==ActiveBullet.end()) break;
		}
	
	list<Ast *>::iterator lia;
	for(lia=ActiveAst.begin();lia!=ActiveAst.end();++lia)
		{
		 (*lia)->Update(dt);
			Clip((*lia)->p);
			AstShipCollision(*lia);
			if((*lia)->active==false)
		 {
			 GameObj *todel=*lia;
			 lia=ActiveAst.erase(lia);
			 delete todel;
		 }
		 if(lia==ActiveAst.end()) break;
		}

	list<GameObj *>::iterator li;
	for(li=ActiveObj.begin();li!=ActiveObj.end();++li)
		{
		 (*li)->Update(dt);
			Clip((*li)->p);
		 if((*li)->active==false)
		 {
			 GameObj *todel=*li;
			 li=ActiveObj.erase(li);
			 delete todel;
		 }
		 if(li==ActiveObj.end()) break;
		}

}

void GameSession::Shoot()
{
 S->Shoot(ActiveBullet);
}

void GameSession::Start()
{
	Exp::LightOffAll();
	S=new Ship3D("newastro.rawc");
	Ast *a;
	for(int i=0;i<10;i++)
		{
			a=new Ast3DFract("tetra.raw",5*(1+rand()%4));
			a->p.x()=drand48()*SpaceSize*2 - SpaceSize;	
			a->p.y()=drand48()*SpaceSize*2 - SpaceSize;
			a->p.z()=0;
			a->v.x()=drand48()*40 - 20;	
			a->v.y()=drand48()*40 - 20;
			a->v.z()=0;
			a->av=drand48()*70;
			a->axis=Normalize(Point3f(drand48(),drand48(),drand48()));
			ActiveAst.push_back(a);
		}
}
void GameSession::AstShipCollision(Ast *a)
{
	if(S->active && a->Collide(S)) // Asteroid - Ship Collision
			{
				S->active=false;
				deadpause=2000;
				Exp *e =new Exp();
				e->p=(a->p+S->p)/2;
				ActiveObj.push_back(e);	
			}
}

void GameSession::BulletAstCollision(Bullet *b)
{
	if(!b->active) return;
	list<Ast *>::iterator lia;
	for(lia=ActiveAst.begin();lia!=ActiveAst.end();++lia)
		if((*lia)->active && (*lia)->Collide(b))
			{
				(*lia)->active=false;
				b->active=false;
				Exp *e =new Exp();
				e->p=(*lia)->p;
				e->LightOn();
				ActiveObj.push_back(e);	
				if((*lia)->size>5)
				{
					Ast *a;
					Point3f nv;
					a=new Ast3DFract("tetra.raw",(*lia)->size-5);
					nv.x()=drand48()*40 - 20;	
			    nv.y()=drand48()*40 - 20;
					nv.z()=0;
					a->av=drand48()*70;
					a->axis=Normalize(Point3f(drand48(),drand48(),drand48()));
					a->p=(*lia)->p;
					a->v=(*lia)->v+nv;
					ActiveAst.push_front(a);
					a=new Ast3DFract("tetra.raw",(*lia)->size-5);
					a->av=drand48()*70;
					a->axis=Normalize(Point3f(drand48(),drand48(),drand48()));
					a->p=(*lia)->p;
					a->v=(*lia)->v-nv;
					ActiveAst.push_front(a);
				}
				return;
			}
}